; ROUTINE TO HANDLE OUTPUT FROM A CIRCULAR BUFFER
;
; Function:
;       This routine handles output from a circular buffer. It checks to
;       see if there is enough bytes in the buffer to send to disk. If there
;       are not enough then it returns without action. If thereis then it sends
;       a b;ock of bytes to an opened file using WRITEFILE routine.
;  Input:
;       BX contains the file handle. Input is from circular buffer.
;  Output:
;       Output is to a file. If the buffer gets nearly empty then a control Q
;       is sent. If write failed, carry is set and error code returned.
;       If a control Z is detected, carry is set and error code 19(end of file)
;  Registers used:
;       AX SI aremodified. AX = exception code. SI = buffer pointer. BX = handle
;  Segments referenced:
;       On entry the data segment must contain the circular buffer and
;       the variables CBUFF, GAUGE, & LASTBYTE
;
outcbuff        proc    far
;
; check for enough characters in the buffer
        cmp     gauge,blksize   ; any characters ?
        jl      outcbuffexit    ; exit if no characters
;
; enpty the buffer
        call    comoff          ; hold the com line
        call    flushit         ; flush the buffer
        je      outcbuffret     ; error ?
;
outcbuffexit:
        clc                     ; cleary carry - no exception
outcbuffret:
        ret
;
outcbuff        endp
;
